/*	The implementation of the PerfCube class.
 *
 *	Created by :    Bala'zs Gero"fi (kedli@inf.elte.hu)
 *                      Ga'bor Bala'zs (bege@inf.elte.hu)       [2003.05.16]
 */

#include <iostream>
#include <cmath>
#include "perfrect.h"
#include "perfcube.h"


PerfCube::PerfCube()
{
  for(int z = 0; z < 16; ++z)
    for(int y = 0; y < 64; ++y)
      for(int x = 0; x < 64; ++x) {
	cube[x][y][z] = new PerfRect();

	// mul and alter the rectangle
	cube[x][y][z]->mul(4);
	cube[x][y][z]->alter( z*64*64 + y*64 + x);
      }
}

PerfCube::~PerfCube()
{
}

int PerfCube::at(int x, int y, int z)
{
     return(cube[x / 4][y / 4][z / 16]->at(x % 4, y % 4, z % 16));
}

bool PerfCube::testing()
{
  const int db = (int)pow(8,8);   // the number of subcubes in the main cube
  int *using_table = new int[db]; // the usage of the subcubes
  int value;

  for (int i = 0; i < db; ++i ) using_table[i]=0;

  for ( int kz = 0; kz < 256; ++kz )
    for ( int ky = 0; ky < 256; ++ky )
      for ( int kx = 0; kx < 256; ++kx ) {

	// get the hash value of the subcube pointed by (kx,ky,kz)
	value = 0;      
	for ( int z = 0; z < 2; ++z )
	  for ( int y = 0; y < 2; ++y )
	    for ( int x = 0; x < 2; ++x )
		value = value * 8 + at((kx+x)%256,(ky+y)%256,(kz+z)%256);
    
	++using_table[value];
      }

  // check the result
  for ( int i = 0; i < db; ++i ) if ( using_table[i] != 1 ) return false;
  return true;
}

void PerfCube::print(ostream &out)
{
  for(int z = 0; z < 256; ++z) {
    for(int y = 0; y < 256; ++y) {

      for(int x = 0; x < 256; ++x)
	out << at(x,y,z) << ' ';

      out << endl;
    }

    out << endl;
  }
}

void PerfCube::printtex(ostream &out)
{
  out << "\\documentclass[10pt,a3paper]{article}" << endl;
  out << "\\begin{document}" << endl;
  out << "\\begin{flushleft}" << endl;

  for(int z = 0; z < 256; ++z) {
    for(int y = 0; y < 256; ++y) {

      out << "\\mbox{";
      for(int x = 0; x < 256; ++x)
	out << at(x,y,z) << ' ';
      out << "}" << endl;

    }

    out << "\\newpage" << endl;
  }

  out << "\\end{flushleft}" << endl;
  out << "\\end{document}" << endl;
}

